import pytest_requirements

project = "pytest_requirements"
copyright = "CTAO"
author = "CTAO Computing Department"
version = pytest_requirements.__version__
# The full version, including alpha/beta/rc tags.
release = version


extensions = [
    "sphinx.ext.intersphinx",
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "numpydoc",
    "sphinx_changelog",
]

exclude_patterns = ["changes"]
default_role = "py:obj"

intersphinx_mapping = {
    "python": ("https://docs.python.org/3/", None),
    "pytest": ("https://docs.pytest.org/en/stable/", None),
}

html_theme = "ctao"
html_theme_options = dict(
    navigation_with_keys=False,
    switcher=dict(
        json_url="http://cta-computing.gitlab-pages.cta-observatory.org/common/pytest-requirements/versions.json",  # noqa: E501
        version_match="latest" if ".dev" in version else f"v{version}",
    ),
    navbar_center=["version-switcher", "navbar-nav"],
)

html_static_path = []



"""pytest plugin providing markers to link tests to requirements and usecases."""

from ._version import __version__


def pytest_configure(config):
    """Register our custom markers."""
    config.addinivalue_line(
        "markers",
        "verifies_requirement(requirement_id): Mark this test as verification for a requirement",
    )
    config.addinivalue_line(
        "markers",
        "verifies_usecase(usecase_id): Mark this test as verification for a usecase",
    )


def pytest_collection_modifyitems(session, config, items):
    """Customize test collection.

    - Make sure requirement/usecase marker information is written to junit xml.
    """
    for item in items:
        for type_ in ("requirement", "usecase"):
            for marker in item.iter_markers(name=f"verifies_{type_}"):
                item.user_properties.append((f"{type_}_id", marker.args[0]))


__all__ = [
    "__version__",
]



import pytest


@pytest.mark.verifies_requirement("B-DPPS-0123")
def test_single_requirement():
    pass


@pytest.mark.verifies_requirement("B-DPPS-0123")
@pytest.mark.verifies_requirement("B-DPPS-0124")
@pytest.mark.verifies_requirement("B-DPPS-0125")
def test_multiple_requirements():
    pass


@pytest.mark.verifies_usecase("UC-130-1.2")
def test_usecase():
    pass


@pytest.mark.verifies_usecase("UC-130-1.2.1")
@pytest.mark.verifies_usecase("UC-130-1.2.2")
def test_multiple_usecases():
    pass


@pytest.mark.verifies_usecase("UC-130-1.2.1")
@pytest.mark.verifies_usecase("UC-130-1.2.2")
@pytest.mark.verifies_requirement("B-DPPS-0123")
def test_mixed():
    pass



from xml.etree import ElementTree as ET

EXPECTED = {
    "test_single_requirement": {
        "requirements": ["B-DPPS-0123"],
        "usecases": [],
    },
    "test_multiple_requirements": {
        "requirements": ["B-DPPS-0123", "B-DPPS-0124", "B-DPPS-0125"],
        "usecases": [],
    },
    "test_usecase": {
        "requirements": [],
        "usecases": ["UC-130-1.2"],
    },
    "test_multiple_usecases": {
        "requirements": [],
        "usecases": ["UC-130-1.2.1", "UC-130-1.2.2"],
    },
    "test_mixed": {
        "requirements": ["B-DPPS-0123"],
        "usecases": ["UC-130-1.2.1", "UC-130-1.2.2"],
    },
}


def collect_markers(test_case):
    markers = {"requirements": [], "usecases": []}
    for prop in test_case.iter("property"):
        if prop.attrib["name"] == "requirement_id":
            markers["requirements"].append(prop.attrib["value"])
        elif prop.attrib["name"] == "usecase_id":
            markers["usecases"].append(prop.attrib["value"])
    return markers


def test_plugin(pytester, tmp_path):
    pytester.copy_example("conftest.py")
    pytester.copy_example("test_requirements.py")

    report_path = tmp_path / "report.xml"
    result = pytester.runpytest(f"--junit-xml={report_path}")

    result.assert_outcomes(passed=5)

    # make sure the marker information is in the junit xml
    tree = ET.parse(report_path)

    test_cases = {
        test_case.attrib["name"]: test_case for test_case in tree.iter("testcase")
    }
    assert len(test_cases) == 5

    for key, expected in EXPECTED.items():
        markers = collect_markers(test_cases[key])
        for type_, expected_ids in expected.items():
            ids = markers[type_]
            assert len(ids) == len(expected_ids)
            assert set(ids) == set(expected_ids)


def test_version():
    from pytest_requirements import __version__

    assert __version__ != "0.0.0"
